//  *****************************************************************************
//   						main.c
// 
//              Demonstracny program pre dosku KTK ARM7
//
//              Ovladanie jednosmerneho motora - H mostik
//
//  *****************************************************************************

// Hlavickove subory
#include "board.h"

// Pocet krokov casovaca na 1 us
#define STEPS   ((unsigned int) ((MCK/8) + 500000) / 1000000L)

// Ovladanie tranzistorov T3 a T4
#define T3	1 << 22		// PA22 - baza tranzistora T5, ktory ovlada tranzistor T3
#define T4	1 << 23		// PA23 - baza tranzistora T6, ktory ovlada tranzistor T4

//  *****************************************************
//  Mikrosekundove oneskorenie (max. cca 10 000)
//  *****************************************************
void delay(unsigned int us)
{
  AT91C_BASE_PMC->PMC_PCER = (1 << AT91C_ID_TC0);  // povolenie hodin pre TC0
  AT91C_BASE_TC0->TC_CMR = 1;                      // volba hodin (MCK/8)
  AT91C_BASE_TC0->TC_RC = us * STEPS;              // nastavenie porovnavacieho registra C
  AT91C_BASE_TC0->TC_CCR = AT91C_TC_CLKEN |        // povolenie hodin
                           AT91C_TC_SWTRG;         // restart casovaca

  // cakanie na ukoncenie oneskorenia
  // ked je nastaveny compare C flag, je hotovo
  while ((AT91C_BASE_TC0->TC_SR & AT91C_TC_CPCS) == 0) asm ("nop");
    
  AT91C_BASE_PMC->PMC_PCDR = (1 << AT91C_ID_TC0);  // zakazanie hodin pre TC0
}

//  *******************************************************
//                Hlavny program
//  *******************************************************
int main(void) 
{ int ii, stav;

  //******************* Uvodna inicializacia *******************

  // Povolenie Resetu - minimalna dlzka 960 us
  AT91C_BASE_RSTC->RSTC_RMR = AT91C_RSTC_URSTEN | (0x4<<8) | (unsigned int)(0xA5<<24);
	
  //******************* Inicializacia portu A ******************

  // Povolenie hodin pre PIOA
  AT91C_BASE_PMC->PMC_PCER = (unsigned int) 1 << AT91C_ID_PIOA; 
	
  // konfiguracia jednotlivych pinov
  // vstupne piny
  AT91C_BASE_PIOA->PIO_ODR = S2;
  // vystupne piny
  AT91C_BASE_PIOA->PIO_OER = T3 | T4;
  // povolenie Pull-Up rezistoru
  AT91C_BASE_PIOA->PIO_PPUER = S2;
  // povolenie vsetkych nakonfigurovanych pinov
  AT91C_BASE_PIOA->PIO_PER = T3 | T4 | S2;

  // tranzistory su rozpojene
  AT91C_BASE_PIOA->PIO_CODR = T3 | T4;

  //******************* Nastavenie PWM ******************
  // Konfiguracia PIO radica do rezimu periferie PWM (kanal PWM0 a PWM1)
  // PIO ma zakazany pristup na dany pin
  AT91C_BASE_PIOA->PIO_PDR = AT91C_PA0_PWM0 | AT91C_PA1_PWM1;
  // povolenie periferie A
  AT91C_BASE_PIOA->PIO_ASR = AT91C_PA0_PWM0 | AT91C_PA1_PWM1;
  // Povolenie hodin pre PWM
  AT91C_BASE_PMC->PMC_PCER = (unsigned int) 1 << AT91C_ID_PWMC;
  // zakazanie vsetkych kanalov
  AT91C_BASE_PWMC->PWMC_DIS = 0x0f;
  // nastavenie modu pre PWM0 a PWM1 - zarovnanie signalu vlavo,  updatuje sa Duty Cycle
  AT91C_BASE_PWMC->PWMC_CH[0].PWMC_CMR =  0x6 | AT91C_PWMC_CPOL;  // volba hodin - MCK/64, polarita = 1 (signal zacina v 1)
  AT91C_BASE_PWMC->PWMC_CH[1].PWMC_CMR =  0x6 | AT91C_PWMC_CPOL;  // volba hodin - MCK/64, polarita = 1 (signal zacina v 1)
  // nastavenie periody: X*64/MCK = 10002*64/47923200 = 0,0133s (cca 75 Hz)
  AT91C_BASE_PWMC->PWMC_CH[0].PWMC_CPRDR = 10002;
  AT91C_BASE_PWMC->PWMC_CH[1].PWMC_CPRDR = 10002;

  //******************* Hlavna cast programu *******************

  stav = 0;
  // nekonecna slucka
  while(1)
  { // cakanie na stlacenie S2  
    while(AT91C_BASE_PIOA->PIO_PDSR & S2)     // kontrola, ci je S2 v log.1
      delay(200);                             // cakanie kvoli potlaceniu zakmitov
    ii = 0;
    // cakanie na pustenie S2
    while(!(AT91C_BASE_PIOA->PIO_PDSR & S2))  // kontrola, ci je S2 v log.0
    { ii++;
      delay(200);                             // cakanie kvoli potlaceniu zakmitov
    }

    // rozlisenie kratkeho a dlheho stlacenia
    if (ii < 2000)          // kratke stlacenie
    { if (stav < 10)
        stav++;
    }
    else                    // dlhe stlacenie
    { if (stav > -10)
        stav--;
    }
    // odpojenie motora
    AT91C_BASE_PIOA->PIO_CODR = T3 | T4;
    // zakazanie kanalov PWM0 a PWM1 (vystupy su nulove)
    AT91C_BASE_PWMC->PWMC_DIS = 3;

    // uprava koeficientu plnenia a volba smeru tocenia
    if (stav < 0)
    { AT91C_BASE_PIOA->PIO_SODR = T4;
      AT91C_BASE_PWMC->PWMC_CH[0].PWMC_CDTYR = 1 - stav*1000;
      AT91C_BASE_PWMC->PWMC_ENA = 1;       // povolenie PWM0
    }
    else if (stav > 0)
    { AT91C_BASE_PIOA->PIO_SODR = T3;
      AT91C_BASE_PWMC->PWMC_CH[1].PWMC_CDTYR = 1 + stav*1000;
      AT91C_BASE_PWMC->PWMC_ENA = 2;       // povolenie PWM1
    }
  }
}
